@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import os
 import collections
 import inspect
 import time
@@ -33,6 +34,7 @@ def create(drivername, *args, **kwargs):
         msg = "drivername should be one of [IE, Opera, Chrome, Firefox](case-insentive). given {0}".format(drivername)
         raise ValueError(msg)
 
+
 def connect(drivername, executor, custom_capabilities=None, **kwargs):
     if not isinstance(drivername, str):
         msg = "drivername should be an instance of string. given {0}".format(type(drivername))
@@ -50,7 +52,7 @@ def connect(drivername, executor, custom_capabilities=None, **kwargs):
                     'android': DesiredCapabilities.ANDROID}
     dname = drivername.lower()
     if dname in capabilities:
-        capability = capabilities[drivername]
+        capability = capabilities[dname]
         custom_capabilities = custom_capabilities or {}
         for key in custom_capabilities:
             capability[key] = custom_capabilities[key]
@@ -63,12 +65,14 @@ def connect(drivername, executor, custom_capabilities=None, **kwargs):
         msg = "drivername should be one of [IE, Opera, Chrome, Firefox](case-insentive). given {0}".format(drivername)
         raise ValueError(msg)
 
+
 def _is_wrappable(obj):
     if isinstance(obj, WebDriver) or isinstance(obj, WebElement):
         return True
     else:
         return False
 
+
 def _chainreact(__getattr__):
     def containment(*methodname):
         def wrap_or_else(obj):
@@ -76,6 +80,7 @@ def _chainreact(__getattr__):
                 return SeleniumWrapper(obj)
             else:
                 return obj
+
         self, methodobj = __getattr__(*methodname)
         if inspect.isroutine(methodobj):
             def reaction(*realargs):
@@ -83,28 +88,31 @@ def _chainreact(__getattr__):
                 # for side-effective method(append, ...)
                 result = result if result is not None else self
                 return wrap_or_else(result)
+
             return reaction
         else:
             return wrap_or_else(methodobj)
+
     return containment
 
-class SeleniumWrapper(object):
 
+class SeleniumWrapper(object):
     def __init__(self, driver):
         if _is_wrappable(driver):
-            self._driver = driver
+            self._wrapped = driver
+            self._timeout = 5
         else:
             msg = "2nd argument should be an instance of WebDriver or WebElement. given {0}.".format(type(driver))
             raise TypeError(msg)
 
     @property
     def unwrap(self):
-        return self._driver
+        return self._wrapped
 
     @property
     def parent(self):
-        if isinstance(self._driver, WebElement):
-            return self.xpath("./parent::node()", timeout=2)
+        if isinstance(self._wrapped, WebElement):
+            return self.xpath("./parent::node()", timeout=self._timeout)
         else:
             raise AttributeError("'WebDriver' object has no attribute 'parent'")
 
@@ -116,36 +124,47 @@ class SeleniumWrapper(object):
 
     @property
     def alert(self):
-        timeout = time.time() + 2
+        timeout = time.time() + self._timeout
         while time.time() < timeout:
             try:
-                alert = self._driver.switch_to_alert()
+                alert = self._wrapped.switch_to_alert()
                 alert.text
                 return alert
             except NoAlertPresentException:
-                time.sleep(0.1)
+                time.sleep(0.2)
         msg = "Wait for alert to be displayed for 2 seconds, but it was not displayed."
         raise NoAlertPresentException(msg)
 
+    def _settimeout(self, timeout):
+        if isinstance(timeout, int):
+            self._timeout = timeout
+        else:
+            raise AttributeError('int please.')
+
+    def _gettimeout(self):
+        return self._timeout
+
+    timeout = property(_gettimeout, _settimeout)
+
     def __getattribute__(self, name):
         return object.__getattribute__(self, name)
 
     @_chainreact
     def __getattr__(self, name):
-        return self._driver, getattr(self._driver, name)
+        return self._wrapped, getattr(self._wrapped, name)
 
     @property
     def current_url(self):
-        self.by_tag("body", 1)
-        return self._driver.current_url
+        self.by_tag("body", self._timeout)
+        return self._wrapped.current_url
 
     def _is_selectable(self):
         return self.unwrap.tag_name == 'select'
 
     def _is_stopping(self, interval):
-        before = (self._driver.location['x'], self._driver.location['y'])
+        before = (self._wrapped.location['x'], self._wrapped.location['y'])
         time.sleep(interval)
-        after = (self._driver.location['x'], self._driver.location['y'])
+        after = (self._wrapped.location['x'], self._wrapped.location['y'])
         return before[0] == after[0] and before[1] == after[1]
 
     def _wait_until_stopping(self, timeout, interval):
@@ -156,36 +175,36 @@ class SeleniumWrapper(object):
             else:
                 time.sleep(interval)
         if not self._is_stopping(interval):
-            raise WebDriverException("Element is not stably displayed for {sec} seconds.".format(sec=timeout))
+            raise WebDriverException("Element was not stably displayed for {sec} seconds.".format(sec=timeout))
 
     def _wait_until_clickable(self, timeout, interval):
         err_messages = []
         endtime = time.time() + timeout
         while True:
             try:
-                self._driver.click()
+                self._wrapped.click()
                 break
             except WebDriverException as e:
                 err_messages.append(e.msg.split(":")[-1].strip())
             time.sleep(interval)
             if (time.time() > endtime):
                 if err_messages:
-                    template = ("Wait for elemtent to be clickable for {sec} seconds, ",
+                    template = ("Waited for element to be clickable for {sec} seconds, ",
                                 "but clicked other elements. {err}")
                     msg = "".join(template).format(sec=timeout, err=err_messages)
                     raise WebDriverException(msg)
 
     def _wait_until_displayed(self, timeout, interval):
         try:
-            WebDriverWait(self._driver, timeout, interval).until(lambda d: d.is_displayed())
+            WebDriverWait(self._wrapped, timeout, interval).until(lambda d: d.is_displayed())
         except TimeoutException:
-            template = ("Wait for elemtent to be displayed for {sec} seconds, ",
+            template = ("Waited for element to be displayed for {sec} seconds, ",
                         "but <{target} ...> was not displayed:: <{dumped}>")
-            msg = "".join(template).format(sec=timeout, target=self._driver.tag_name, dumped=self._dump())
+            msg = "".join(template).format(sec=timeout, target=self._wrapped.tag_name, dumped=self._dump())
             raise ElementNotVisibleException(msg)
 
     def _dump(self):
-        element = self._driver
+        element = self._wrapped
         info = {"visibility": element.value_of_css_property("visibility"),
                 "display": element.value_of_css_property("display"),
                 "height": element.value_of_css_property("height"),
@@ -195,41 +214,72 @@ class SeleniumWrapper(object):
         dumped = " ".join(["{k}:{v}".format(k=k, v=info[k]) for k in info])
         return dumped
 
-    def click(self, timeout=3, presleep=0, postsleep=0):
-        if isinstance(self._driver, WebElement):
+    def attr(self, name):
+        if isinstance(self._wrapped, WebElement):
+            return self._wrapped.get_attribute(name)
+        else:
+            raise AttributeError("This is WebDriver wrapped object.")
+
+    def click(self, timeout=None, presleep=0, postsleep=0):
+        timeout = timeout or self._timeout
+        if isinstance(self._wrapped, WebElement):
             try:
                 if presleep:
                     time.sleep(presleep)
                 self._wait_until_stopping(timeout, 0.1)
-                self._wait_until_displayed(timeout, 0.2)
+                self._wait_until_displayed(timeout, 0.3)
                 self._wait_until_clickable(timeout, 0.3)
                 if postsleep:
                     time.sleep(postsleep)
             except Exception as e:
                 raise e
 
-    def waitfor(self, type, target, eager=False, timeout=3):
-        if eager:
-            types = {"id":lambda d: d.find_elements_by_id(target),
-                     "name":lambda d: d.find_elements_by_name(target),
-                     "xpath":lambda d: d.find_elements_by_xpath(target),
-                     "link_text":lambda d: d.find_elements_by_link_text(target),
-                     "partial_link_text":lambda d: d.find_elements_by_partial_link_text(target),
-                     "tag":lambda d: d.find_elements_by_tag_name(target),
-                     "class":lambda d: d.find_elements_by_class_name(target),
-                     "css":lambda d: d.find_elements_by_css_selector(target), }
+    def scroll_to(self, x, y):
+        if isinstance(self._wrapped, WebDriver):
+            return self._wrapped.execute_script("window.scrollTo({:d}, {:d})".format(x, y))
         else:
-            types = {"id":lambda d: d.find_element_by_id(target),
-                     "name":lambda d: d.find_element_by_name(target),
-                     "xpath":lambda d: d.find_element_by_xpath(target),
-                     "link_text":lambda d: d.find_element_by_link_text(target),
-                     "partial_link_text":lambda d: d.find_element_by_partial_link_text(target),
-                     "tag":lambda d: d.find_element_by_tag_name(target),
-                     "class":lambda d: d.find_element_by_class_name(target),
-                     "css":lambda d: d.find_element_by_css_selector(target), }
+            raise AttributeError("This is WebElement wrapped object.")
+
+    def scroll_by(self, x, y):
+        if isinstance(self._wrapped, WebDriver):
+            return self._wrapped.execute_script("window.scrollBy({:d}, {:d})".format(x, y))
+        else:
+            raise AttributeError("This is WebElement wrapped object.")
+
+    def scroll_into_view(self, jq_identifier, align_with_top=True):
+        if isinstance(self._wrapped, WebDriver):
+            if self._wrapped.execute_script("try{return $;}catch(e){}"):
+                script_template = "try{{$('{0}') && $('{0}')[0].scrollIntoView({1})}}catch(e){{}}"
+                script = script_template.format(jq_identifier, 'true' if align_with_top else 'false')
+                self._wrapped.execute_script(script)
+            else:
+                raise AttributeError("You must load jquery library.")
+        else:
+            raise AttributeError("This is WebElement wrapped object.")
+
+    def waitfor(self, type, target, eager=False, timeout=None):
+        timeout = timeout or self._timeout
+        if eager:
+            types = {"id": lambda d: d.find_elements_by_id(target),
+                     "name": lambda d: d.find_elements_by_name(target),
+                     "xpath": lambda d: d.find_elements_by_xpath(target),
+                     "link_text": lambda d: d.find_elements_by_link_text(target),
+                     "partial_link_text": lambda d: d.find_elements_by_partial_link_text(target),
+                     "tag": lambda d: d.find_elements_by_tag_name(target),
+                     "class": lambda d: d.find_elements_by_class_name(target),
+                     "css": lambda d: d.find_elements_by_css_selector(target), }
+        else:
+            types = {"id": lambda d: d.find_element_by_id(target),
+                     "name": lambda d: d.find_element_by_name(target),
+                     "xpath": lambda d: d.find_element_by_xpath(target),
+                     "link_text": lambda d: d.find_element_by_link_text(target),
+                     "partial_link_text": lambda d: d.find_element_by_partial_link_text(target),
+                     "tag": lambda d: d.find_element_by_tag_name(target),
+                     "class": lambda d: d.find_element_by_class_name(target),
+                     "css": lambda d: d.find_element_by_css_selector(target), }
         finder = types[type]
         try:
-            result = WebDriverWait(self._driver, timeout).until(finder)
+            result = WebDriverWait(self._wrapped, timeout).until(finder)
             if eager and len(result):
                 return SeleniumContainerWrapper(result)
             elif _is_wrappable(result):
@@ -237,49 +287,49 @@ class SeleniumWrapper(object):
             else:
                 return result
         except TimeoutException:
-            template = ("Wait for elemtent to appear for {sec} seconds, ",
+            template = ("Waited for element to appear for {sec} seconds, ",
                         "but {type}:{target} didn't appear.")
             msg = "".join(template).format(sec=timeout, type=type, target=target)
             raise NoSuchElementException(msg)
 
-    def xpath(self, target, eager=False, timeout=3):
+    def xpath(self, target, eager=False, timeout=None):
         return self.waitfor("xpath", target, eager, timeout)
 
-    def css(self, target, eager=False, timeout=3):
+    def css(self, target, eager=False, timeout=None):
         return self.waitfor("css", target, eager, timeout)
 
-    def by_tag(self, tag, eager=False, timeout=3, **attributes):
+    def by_tag(self, tag, eager=False, timeout=None, **attributes):
         subjects = ["@{key}='{val}'".format(key=k, val=attributes[k]) for k in attributes]
         subject = " and ".join(subjects)
         xpath = ".//{tag}[{subject}]".format(tag=tag, subject=subject) if subject else ".//{tag}".format(tag=tag)
         return self.waitfor('xpath', xpath, eager, timeout)
 
-    def by_text(self, text, tag="*", partial=False, eager=False, timeout=3):
+    def by_text(self, text, tag="*", partial=False, eager=False, timeout=None):
         if partial:
             return self.xpath(".//{tag}[contains(text(), '{text}')]".format(tag=tag, text=text), eager, timeout)
         return self.xpath(".//{tag}[text()='{text}']".format(tag=tag, text=text), eager, timeout)
 
-    def by_class(self, target, eager=False, timeout=3):
+    def by_class(self, target, eager=False, timeout=None):
         return self.waitfor("class", target, eager, timeout)
 
-    def by_id(self, target, eager=False, timeout=3):
+    def by_id(self, target, eager=False, timeout=None):
         return self.waitfor("id", target, eager, timeout)
 
-    def by_name(self, target, eager=False, timeout=3):
+    def by_name(self, target, eager=False, timeout=None):
         return self.waitfor("name", target, eager, timeout)
 
-    def by_linktxt(self, target, eager=False, timeout=3, partial=False):
+    def by_linktxt(self, target, eager=False, timeout=None, partial=False):
         if partial:
-            return self.waitfor("partial_link_text", target, eager, timeout=3)
+            return self.waitfor("partial_link_text", target, eager, timeout=None)
         else:
             return self.waitfor("link_text", target, eager, timeout)
 
-    def href(self, partialurl=None, eager=False, timeout=3):
+    def href(self, partialurl=None, eager=False, timeout=None):
         if partialurl:
             return self.xpath(".//a[contains(@href, '{0}')]".format(partialurl), eager, timeout)
         return self.xpath(".//a", eager, timeout)
 
-    def img(self, alt=None, ext=None, eager=False, timeout=3):
+    def img(self, alt=None, ext=None, eager=False, timeout=None):
         options = []
         if alt:
             options.append("@alt='{0}'".format(alt))
@@ -289,18 +339,18 @@ class SeleniumWrapper(object):
         xpath = ".//img" + "[{0}]".format(option) if option else ".//img"
         return self.xpath(xpath, eager, timeout)
 
-    def button(self, value, eager=False, timeout=3):
-        return self.xpath("//input[@type='submit' or @type='button' and @value='{0}']".format(value), eager, timeout)
+    def button(self, value, eager=False, timeout=None):
+        return self.xpath(".//input[@type='submit' or @type='button' and @value='{0}']".format(value), eager, timeout)
 
-    def checkbox(self, eager=False, timeout=3, **attributes):
+    def checkbox(self, eager=False, timeout=None, **attributes):
         attributes["type"] = "checkbox"
         return self.by_tag("input", eager, timeout, **attributes)
 
-    def radio(self, eager=False, timeout=3, **attributes):
+    def radio(self, eager=False, timeout=None, **attributes):
         attributes["type"] = "radio"
         return self.by_tag("input", eager, timeout, **attributes)
 
-    def select(self, eager=False, timeout=3, **attributes):
+    def select(self, eager=False, timeout=None, **attributes):
         selected = self.by_tag("select", eager, timeout, **attributes)
         if isinstance(selected, SeleniumWrapper) and selected._is_selectable():
             return Select(selected.unwrap)
@@ -309,13 +359,13 @@ class SeleniumWrapper(object):
             selected._iterable = [Select(element) for element in iterable if element.tag_name == 'select']
             return selected
         else:
-            template = ("Wait for elemtent to appear for {sec} seconds, ",
+            template = ("Waited for element to appear for {sec} seconds, ",
                         "but select:{attr} didn't appear.")
             msg = "".join(template).format(sec=timeout, attr=attributes)
             raise NoSuchElementException(msg)
 
-class SeleniumContainerWrapper(object):
 
+class SeleniumContainerWrapper(object):
     def __init__(self, iterable):
         if not isinstance(iterable, collections.Sequence):
             msg = "2nd argument should be an instance of collections.Sequence. given {0}".format(type(iterable))
@@ -340,6 +390,10 @@ class SeleniumContainerWrapper(object):
         key = key.unwrap if isinstance(key, SeleniumWrapper) else key
         return key in self._iterable
 
+    @property
+    def size(self):
+        return len(self._iterable)
+
     def sample(self, size):
         picked = random.sample(self._iterable, size)
         if isinstance(picked, collections.Sequence):
